import ij.plugin.PlugIn;
import ij.*;
import ij.gui.*;
import ij.io.*;
import java.util.*;
import java.io.*;
import ij.process.*;
import java.awt.*;
import ij.measure.*;
import ij.plugin.frame.*;
import ij.plugin.*;
import ij.plugin.filter.*;
import ij.text.*;
import java.awt.image.*;
import ij.util.*;
import java.text.*;

/**
 * This is a template for a plugin that does not require one image
 * (be it that it does not require any, or that it lets the user
 * choose more than one image in a dialog).
 */
public class PelletAnalysisJava implements PlugIn {
	/**
	 * This method gets called by ImageJ / Fiji.
	 *
	 * @param arg can be specified in plugins.config
	 * @see ij.plugin.PlugIn#run(java.lang.String)
	 */

	static boolean Ren, Com, Ana, Filter, Check, Excel, Outlines, FocusCheck, RenError, skip, Reject;
	static int Col,ShedSet,LowLimit, ImTypeIndex, UpLimit;
	static String[] ImType;
	static String NanPix, FilterPath, CenterName, renFol, inFol, outFol, tFol, shortName, BMT, BMF;
	static ArrayList<String> FolArray;
	static RoiManager rm = RoiManager.getInstance(); //for active RoiManager
	static ResultsTable rt = new ResultsTable(); //for reading results
	static ResultsTable rtout = new ResultsTable(); // for setting results
	static ResultsTable rtpart = new ResultsTable();
	static ImagePlus Original;
	static WaitForUserDialog wfud;
	static int Identifier;
	static TextWindow RW;
	@Override
	public void run(String arg) {
		IJ.run("Input/Output...", "jpeg=85 gif=-1 file=.csv use_file copy_row save_column");
		IJ.log("\\Clear");
		System.gc();
		Calendar cal = Calendar.getInstance();
        	SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        	IJ.log(sdf.format(cal.getTime()) );
		RW=rt.getResultsWindow();
		if (!(rm==null)){
			rm.close();
		}
		rm=new RoiManager();
		IJ.run("Close All", "");
		Ana=true;
		//call menu dialog
		PelMenu();
		if (!skip){ // goes out of the loop on cancel
			SelFilter();
			SelFolders();
			//for loop for folders
			for (int i=0;i<FolArray.size();i++){
				System.gc();
				cal = Calendar.getInstance();
        			IJ.log(sdf.format(cal.getTime()) );
				//renames all files to allow the mosaic to be build
				if (Ren){
					RenFile(i);
				}
				if (RenError){ // if some files cannot be renamed the folder is skipped.
					IJ.log(""+FolArray.get(i)+ " skipped due to renaming error");
					continue;
				}
				//mosaic
				if (Com){
					ComFile(i);
				}
				//analysis
				if (Ana){
					tFol=FolArray.get(i); // creates the proper folders for analysis
					if (Com){
						outFol=tFol+"\\mosaic";
					} else {
						outFol=tFol;
					}
					new File(tFol+"\\analysed").mkdir();
					new File(tFol+"\\rejected").mkdir();
			
					if (Excel){ // opens excel if the results need to be screened
						String directory= IJ.getDirectory("plugins");
						String ExcF1 ="SParticle\\GraphLinker.xlsm";
						try {
							Desktop.getDesktop().open(new File(directory+ExcF1));
						} catch (IOException err){
							IJ.log(err.getMessage());
							Excel=false;
						} 
					}
						
					File tempFol= new File(outFol); // get list of mosaics
					File[] LOF=tempFol.listFiles(); 
					rt=ResultsTable.getResultsTable();
					rt.reset();// empties results table

					for (int l=0; l<LOF.length;l++){
						String tFileName=LOF[l].toString();
						if ((tFileName.toLowerCase()).endsWith(".tif")){
							rm=MaskObjects (l, tFileName); //after this the roi manager contains all possible objects
							rm=RoiManager.getInstance(); // needed to reanalyze the others
							
							Original.show();
							ImageWindow Orwin=Original.getWindow();
							Orwin.setVisible(false);
							RW=rt.getResultsWindow();
							RW.setVisible(false);
							IJ.runMacroFile(FilterPath,""+Reject);
							
							RW=rt.getResultsWindow();
							if (!(RW==null)){
								RW.setVisible(false);
							}
							Original.hide();
							SortRois();
							DoubleChecker();
							//FocusCheck to be added
							if (FocusCheck){
								FocusChecker();
							}
							//Manual roiManager check
							if (Check){
								wfud = new WaitForUserDialog("Remove unwanted rois from the ROI manager and press OK when done");
								wfud.show();
							}
							//Measure remaining Rois
							MeasureRois();
							//Show and save results
							if(!(RW==null)){
								RW.setVisible(true);
								
							}
							rm.setVisible(true);
							try {
								rtpart.saveAs(outFol+CenterName+".xls");
							} catch (IOException e){
								IJ.log ("Data could not be saved for "+CenterName); 
							}
							rtpart.reset();
						}
					}
				}
			}
			cal = Calendar.getInstance();
			IJ.log(sdf.format(cal.getTime()) );
			if (Ana){
				rtout.show("Results");
				try {
					rtout.saveAs(outFol+"Results.xls");
				} catch (IOException e){
					wfud = new WaitForUserDialog("Results could not be saved, please store them manually");
					wfud.show();
				}
			}
			if (Excel){
				try {
					Desktop.getDesktop().open(new File(outFol+"Results.xls"));
				} catch (IOException err){
					IJ.log(err.getMessage());
					Excel=false;
				} 
			}
			
			IJ.run("Close All", "");
		}
	}

	void AnalyzeFiles(){
		
	}

	void MeasureRois(){
		IJ.run(Original, "Set Scale...", "distance=1 known=0." +NanPix+ " pixel=1 unit=micron");
		Roi[] tRoi=rm.getRoisAsArray(); // needed for BMD and BSD
		IJ.run("Set Measurements...", "area mean standard modal min centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction stack redirect=None decimal=2");
		Identifier+=1000000;
		for (int j=0;j<rm.getCount();j++){
			rtout.incrementCounter();
			rtpart.incrementCounter();
			rt.reset();
			IJ.run(Original, "Measure", ""); // measure before duplicating the pic.
			Roi curRoi=(Roi)tRoi[j].clone();
			Original.setRoi(curRoi);
			IJ.run(Original, "Enlarge...", "enlarge="+((int)(rt.getValue("Feret",0)/10)));
			// if outlines need to be saved
			ImagePlus impT=new Duplicator().run(Original);
			if (Outlines){
				int Offset=(int)(rt.getValue("Feret",0)/10);
				curRoi.setLocation(Offset,Offset);
				curRoi.drawPixels(impT.getProcessor());
				impT=new ImagePlus("outlined",impT.getProcessor());
			}
			IJ.saveAs(impT, "Jpeg", outFol+"analysed\\"+CenterName+"_Pellet_"+(Identifier+j)+".jpg");
			
			rtout.addValue("Title",CenterName); // add the title of the image to the resulttable
			rtout.addValue("Hyperlink","=HYPERLINK(\""+outFol+"analysed\\"+CenterName+"_Pellet_"+(Identifier+j)+".jpg\";\""+(Identifier+j)+"\")");
			rtout.addValue("Area",rt.getValue("Area",0));
			rtout.addValue("Mean",rt.getValue("Mean",0));
			rtout.addValue("Stdev",rt.getValue("StdDev",0));
			rtout.addValue("Round",rt.getValue("Round",0));
			rtout.addValue("Circ.",rt.getValue("Circ.",0));
			rtout.addValue("Max Feret",rt.getValue("Feret",0));
			rtout.addValue("Min Feret",rt.getValue("MinFeret",0));
			rtout.addValue("Perimeter",rt.getValue("Perim.",0));
			rtout.addValue("Elipse Perimeter",rt.getValue("Major",0));
			rtout.addValue("Perimeter ratio",(rt.getValue("Perim.",0)/rt.getValue("Major",0)));
			rtout.addValue("Solidity",rt.getValue("Solidity",0));
			double MN = (2*Math.sqrt(rt.getValue("Area",0))*rt.getValue("Solidity",0)*rt.getValue("Round",0))/(Math.sqrt(Math.PI)*rt.getValue("Feret",0));
			rtout.addValue("Morphological Number",MN);

			rtpart.addValue("Title",CenterName); // add the title of the image to the resulttable
			rtpart.addValue("Hyperlink","=HYPERLINK(\""+outFol+"analysed\\"+CenterName+"_Pellet_"+(Identifier+j)+".jpg\";\""+(Identifier+j)+"\")");
			rtpart.addValue("Area",rt.getValue("Area",0));
			rtpart.addValue("Mean",rt.getValue("Mean",0));
			rtpart.addValue("Stdev",rt.getValue("StdDev",0));
			rtpart.addValue("Round",rt.getValue("Round",0));
			rtpart.addValue("Circ.",rt.getValue("Circ.",0));
			rtpart.addValue("Max Feret",rt.getValue("Feret",0));
			rtpart.addValue("Min Feret",rt.getValue("MinFeret",0));
			rtpart.addValue("Perimeter",rt.getValue("Perim.",0));
			rtpart.addValue("Elipse Perimeter",rt.getValue("Major",0));
			rtpart.addValue("Perimeter ratio",(rt.getValue("Perim.",0)/rt.getValue("Major",0)));
			rtpart.addValue("Solidity",rt.getValue("Solidity",0));
			rtpart.addValue("Morphological Number",MN);
			
			// this part for fractals
			Original.setRoi(curRoi);
			impT=new Duplicator().run(Original);
			curRoi.setLocation(0,0);
			impT=new ImagePlus("Mask",curRoi.getMask());

			FractalBoxCounterForMe FBC=new FractalBoxCounterForMe();
			FBC.run(impT.getProcessor());
			IJ.run(impT, "Find Edges", "");
			FBC.run(impT.getProcessor());
			rtout.addValue("BMD",rt.getValue("D",1));
			rtout.addValue("BSD",rt.getValue("D",2));
			rtout.addValue("Fractal quotient",(rt.getValue("D",1)/rt.getValue("D",2)));

			rtpart.addValue("BMD",rt.getValue("D",1));
			rtpart.addValue("BSD",rt.getValue("D",2));
			rtpart.addValue("Fractal quotient",(rt.getValue("D",1)/rt.getValue("D",2)));

			
			// now for polar circularity
			FloatPolygon RoiData=curRoi.getInterpolatedPolygon();
			float[] xs= RoiData.xpoints;
			float[] ys= RoiData.ypoints;
			double XM=rt.getValue("XM",0);
			double YM=rt.getValue("YM",0);
			double[] Radii = new double[xs.length];
			double totaldist=0;
			for (int k=0;k<xs.length;k++){
				Radii[k]=Math.sqrt(Math.pow((xs[k]-XM),2)+Math.pow((ys[k]-YM),2));
				totaldist+=Radii[k];
			}
			double av=totaldist/xs.length;
			double standarddevationsum=0;
			for (int k=0;k<xs.length;k++){
				standarddevationsum+=Math.pow((Radii[k]-av),2);
			}
			double standarddeviation=Math.sqrt(standarddevationsum/xs.length);
			rtout.addValue("Polar circularity",standarddeviation/av);
			rtpart.addValue("Polar circularity",standarddeviation/av);
			//Fluffyness
			double prevBA=0;
			double BA=rt.getValue("Area",0);
			double BI=rt.getValue("Mean",0);
			double BS=Math.floor(rt.getValue("MinFeret",0));
			double tF=rt.getValue("MinFeret",0);

			if (BS==00) BS++;
			double BT=BA*BI;
			double DM=0;
			double DMin=255;
			double [] BandArr=new double[20];
			double prevBI, DA, DI,DT;
			int maxP=0, minP=0;
			curRoi=(Roi)tRoi[j].clone();
			Original.setRoi(curRoi);
			RoiEnlarger RE=new RoiEnlarger();
			for (int k=0;k<20;k++){
				curRoi=RE.enlarge(curRoi,-BS);			
				Original.setRoi(curRoi);
				//IJ.run(Original, "Enlarge...", "enlarge=-"+BS);
				rt.reset();
				IJ.run(Original, "Measure", "");
				prevBA=BA;
				prevBI=BI;
				BA=rt.getValue("Area",0);
				BI=rt.getValue("Mean",0);
				DA=prevBA-BA;
				if (DA==0){
					DT=prevBI;
					DI=DT/BA;
				} else {
					DT=(prevBA*prevBI)-(BA*BI);
					DI=DT/DA;
				}
				if (DI>DM){
					DM=DI;
					maxP=k+1;
				}
				BandArr[k]=DI;
			}
			for (int k=maxP;k<20;k++){
				if (BandArr[k]<DMin){
					DMin=BandArr[k];
					minP=k+1;	
				}
			}
			//ResultsTotal[1]=#Polar Circ#Fluf size#Fluf ratio#Density Ratio"; //headers for output table
			rtout.addValue("Fluf size",(maxP+1)*tF);
			rtpart.addValue("Fluf size",(maxP+1)*tF);
			try {
				rtout.addValue("Fluf ratio", Math.abs((maxP)/(maxP-minP)));
				rtpart.addValue("Fluf ratio", Math.abs((maxP)/(maxP-minP)));
			} catch (ArithmeticException e){
				rtout.addValue("Fluf ratio", "Infinity");
				rtpart.addValue("Fluf ratio", "Infinity");
			}
			rtout.addValue("Density ratio",((DM-BandArr[0])/(DM-DMin)));			
			rtpart.addValue("Density ratio",((DM-BandArr[0])/(DM-DMin)));			
		}
	}

	void FocusChecker(){
		Roi[] tRoi=rm.getRoisAsArray();
		for (int j=rm.getCount()-1;j>=0;j--){
			rt.reset();
			Roi curRoi=(Roi)tRoi[j].clone();
			Original.setRoi(curRoi);
			ImagePlus impT=new Duplicator().run(Original);
			curRoi.setLocation(0,0);
			impT.setRoi(curRoi);
			IJ.run(impT, "Measure", "");
			int AreaVal=(int)rt.getValue("Area",0);
			int EL, BS;
			if (AreaVal<1000){
				EL=-2;
				BS=4;
			} else if (AreaVal<10000){
				EL=-5;
				BS=10;
			} else if (AreaVal<100000){
				EL=-10;
				BS=20;
			} else {
				EL=-20;
				BS=40;
			}
			IJ.run(impT, "Find Edges", "");
			IJ.run(impT, "Enlarge...", "enlarge=-10");
			IJ.run("Make Band...", "band=20");
			IJ.run(impT, "Measure", "");
			if (rt.getValue("Mean",1)<35){
				rm.select(j);
				rm.runCommand("Delete");
			}
		}
	}

	void DoubleChecker(){
		Roi[] tRoi=rm.getRoisAsArray();
		for (int j=rm.getCount()-1;j>=0;j--){
			Roi curRoi=(Roi)tRoi[j].clone();
			Original.setRoi(curRoi);			
			rt.reset();
			IJ.run("Set Measurements...", "area mean standard modal min centroid center perimeter bounding fit shape feret's integrated median skewness kurtosis area_fraction stack redirect=None decimal=2");
			ImagePlus impT=new Duplicator().run(Original);
			curRoi.setLocation(0,0);
			impT.setRoi(curRoi);
			ImageProcessor IPT=impT.getProcessor();
			IJ.run(impT, "Measure", "");
			int AreaVal=(int)rt.getValue("Area",0);
			IJ.run(impT, "Invert", "");
			IJ.setBackgroundColor(0, 0, 0);
			IPT.fillOutside(impT.getRoi());
			IJ.run(impT, "Select None", "");
			IJ.run(impT, "Gaussian Blur...", "sigma="+(impT.getWidth()/10));
			IJ.run(impT, "Find Maxima...", "noise=0 output=Count exclude");
			int Maxes=(int)rt.getValue("Count",1);
			if (Maxes>1 && AreaVal>50000){
				rm.select(j);
				rm.runCommand("Delete"); 
			}
			impT=null;
		}
	}

	void SortRois(){
		rm.runCommand("Save", ""+outFol+shortName+"_MeasuredRois.zip");
		for (int j=0;j<rm.getCount();j++){
			rm.select(j);
			String ArSt=""+rt.getValue("Area",j);
			while (ArSt.length()<9){
				ArSt="0"+ArSt;
			}
			rm.runCommand("Rename",ArSt);
		}
		rm.runCommand("Sort");
	}

	RoiManager MaskObjects (int FolNum, String tFileName){
	if (rm==null){
		rm=new RoiManager();
		rm.setVisible(false);
		rm=RoiManager.getInstance();
	} else {
		rm.close();
		rm =new RoiManager();
		rm.setVisible(false);
		rm=RoiManager.getInstance();
	}
	Original = IJ.openImage(tFileName);	
	IJ.run(Original, "Set Scale...", "distance=0 known=0 pixel=1 unit=pixel");
	IJ.run(Original, "8-bit", "");
	String tName=Original.getTitle();
	shortName=tName.substring(0,tName.length()-4);
	rt.reset();
	CenterName=shortName;
	Original.setTitle("Original");
	IJ.run("Set Measurements...", "area mean modal redirect=None decimal=2");
	IJ.run(Original, "Select All", "");
	IJ.run(Original, "Measure", "");
	int Bck = (int)rt.getValue("Mode",0);
/*might be Original*/		IJ.run("Set Measurements...", "area mean standard perimeter fit shape integrated area_fraction feret's redirect=None decimal=2");//Measurements are set to include more information (area, mean, standard deviation etc.)!!!!!!!!!!!!!!!!!!!!!!
	if (ImTypeIndex==0){
		IJ.run(Original, "Multiply...", "value="+(66/(double)Bck));
	}
	// this part sets some options to run everything smoothly
	IJ.run("Colors...", "foreground=yellow background=black selection=yellow");
	IJ.run(Original, "Options...", "iterations=1 count=1 black pad edm=Overwrite");

	ImagePlus impA=Original.duplicate();

	if (ImTypeIndex==0){
		IJ.setThreshold(impA, 50, 90);
	} else {
		IJ.setThreshold(impA, 0, 10);
	}
	IJ.run(impA, "Convert to Mask", "");
	IJ.run(impA, "Invert", "");
	ImagePlus impB=FillHoles(impA); // now filling holes on edges

	// can potentially be added to the ##settings menu
	IJ.run(impB, "Dilate", "");
	IJ.run(impB, "Fill Holes", "");
	IJ.run(impB, "Erode", "");
	// this part clears the overly large objects
	IJ.run(impB, "Analyze Particles...", "size="+UpLimit+"-Infinity circularity=0.00-1.00 show=Nothing add in_situ");
	if (!(RW==null)){
		RW.setVisible(false);	
	}
	//rm.getInstance2();
	rm.setVisible(true);
	Roi[] tRoi=rm.getRoisAsArray();
	for (int ii=tRoi.length-1;ii>0;ii--){
		IJ.log(""+ii);
		impB.setRoi(tRoi[ii]);
		IJ.run(impB, "Fill", "slice");
		//rm.runCommand("Delete");
	}
	ImagePlus impC=impB.duplicate();
	IJ.run(impC, "Gaussian Blur...", "sigma=25");
	IJ.setThreshold(impC, 150, 255);
	IJ.run(impC, "Convert to Mask", "");
	ImagePlus impD=impC.duplicate();
	IJ.run(impD, "Watershed", "");
	ImageCalculator ic = new ImageCalculator();
	ImagePlus impG = ic.run("substract create", impC, impD);
	ParticleAnalyzer PA=new ParticleAnalyzer(4104, 0, rt, ShedSet, UpLimit);
	PA.analyze(impG);
	ImagePlus impF = new ImagePlus();
	impF.hide();
	impF = PA.getOutputImage();
	impF.hide();
	//IJ.run(impA, "Gaussian Blur...", "sigma=6");
	//ImagePlus impI = ic.run("and create", impA, impF); // this is the basis for selecting between empty pellets and real seperable objects
	ImagePlus impE = ic.run("add create", impD, impF);
	IJ.run(impE, "Erode", "");
	IJ.run(impE, "Voronoi", "");
	IJ.setThreshold(impE, 1, 255);
	IJ.run(impE, "Convert to Mask", "");
	IJ.run(impE, "Dilate", "");
	ImagePlus impH = ic.run("substract create", impB, impE);
	
	rt.reset();
	//IJ.run(impH, "Analyze Particles...", "size="+ LowLimit+"-Infinity circularity=0.00-1.00 show=Nothing display exclude add");
	if (!(RW==null)){
		RW.setVisible(false);
	}
	PA=new ParticleAnalyzer(2056, 0, rt, LowLimit, UpLimit);
	PA.setRoiManager(rm);
	PA.analyze(impH);
	//rt.show("Tempres");
	rm.runCommand("Save", ""+outFol+shortName+"_allRois.zip");
	rm.setVisible(false);
	/*IJ.log("1");
	impA.flush();
	impH.flush();
	impB.flush();
	impE.flush();
	impD.flush();
	impF.flush();
	impC.flush();
	impG.flush();
	*/
	return rm;
}

	ImagePlus FillHoles(ImagePlus impFill){
		int Hei=impFill.getHeight();
		int Wid=impFill.getWidth();
		IJ.run("Colors...", "foreground=yellow background=white selection=yellow");
		IJ.run("Colors...", "foreground=yellow background=white selection=yellow");
		ImageProcessor IP = impFill.getProcessor();
		ImagePlus retImp= new ImagePlus("Filled",IP);
		IJ.run(retImp, "Canvas Size...", "width="+(Wid+1)+" height="+(Hei+1)+" position=Top-Left");
		IJ.run(retImp, "Fill Holes", "");
		IJ.run(retImp, "Canvas Size...", "width="+(Wid)+" height="+(Hei)+" position=Top-Left");

		IJ.run(retImp, "Canvas Size...", "width="+(Wid+1)+" height="+(Hei+1)+" position=Top-Right");
		IJ.run(retImp, "Fill Holes", "");
		IJ.run(retImp, "Canvas Size...", "width="+(Wid)+" height="+(Hei)+" position=Top-Right");

		IJ.run(retImp, "Canvas Size...", "width="+(Wid+1)+" height="+(Hei+1)+" position=Bottom-Left");
		IJ.run(retImp, "Fill Holes", "");
		IJ.run(retImp, "Canvas Size...", "width="+(Wid)+" height="+(Hei)+" position=Bottom-Left");
		
		IJ.run(retImp, "Canvas Size...", "width="+(Wid+1)+" height="+(Hei+1)+" position=Bottom-Right");
		IJ.run(retImp, "Fill Holes", "");
		IJ.run(retImp, "Canvas Size...", "width="+(Wid)+" height="+(Hei)+" position=Bottom-Right");

		
		IJ.run("Colors...", "foreground=yellow background=black selection=yellow");
		return retImp;

	}

	void ComFile (int FolNum){
		int rowN, nCom, colR, rowR, nCIm, nRIm, CS, RS;
		long MaxRam;
		IJ.run("Close All", "");
		if (Ren){
			inFol=FolArray.get(FolNum)+"\\renamed";
			tFol=FolArray.get(FolNum);
			outFol=tFol+"\\mosaic";
		} else {
			tFol=FolArray.get(FolNum);
			inFol=tFol;
			outFol=tFol+"\\mosaic";
		}
		new File(tFol+"\\mosaic").mkdir();
		File tempFol= new File(inFol);
		File[] LOF=tempFol.listFiles();
		
		rowN=(int)(Math.floor(LOF.length/Col));
		IJ.run("Close All", "");
		
		//This part is to calculate the number of parts the final image will be divided in. For computers with high RAM availability this might result in 1 big image while for machines
		//with low RAM availability it might be created by dividing the entire slide area in 4 and creating 4 mosaic images. In which case final column and row numbers will be 2.
		MaxRam=(int)IJ.maxMemory();
		nCom=(int)(Math.floor(MaxRam/50000000));
		if (nCom>Col){
			nCom=Col;
		}
		
		colR=Col%nCom;
		rowR=rowN%nCom;
		if (colR==0){
			nCIm=(Col/nCom);
		} else {
			nCIm=((Col-colR)/nCom)+1;
		}

		if (rowR==0){
			nRIm=(rowN/nCom);
		} else {
			nRIm=((rowN-rowR)/nCom)+1;
		}
		//If the slide are was devided by 45x45 images and if RAM availability calculation enables 20x20 images to be the max, then final row and colum number will be 3 and 9 mosaic...
		//...images will be created (4x(20x20),2x(5x20),2x(20x5),1x(5x5)).
		//1-1, 1-2, 2-1, 2-2 (20x20)
		//1-3, 2-3 (5x20)
		//3-1, 3-2 (20x5)
		//3-6 (5x5)

		for (int j=1;j<nRIm+1;j++){
			for (int jj=1;jj<nCIm+1;jj++){
				if (jj==nCIm && colR!=0){ //If the loop is in the final Col and there are remainders...
					CS=colR;
				} else {
					CS=nCom;
				}
				
				if (j==nRIm && rowR !=0){ //If the loop is in the final Row and there are remainders...
					RS=rowR;
				} else {
					RS=nCom;
				}
				ImagePlus imp = new ImagePlus();
				imp = IJ.createImage("Center", "8-bit black", 1000+(CS-1)*900, 1000+(RS-1)*900, 1);
				ImageProcessor impP=imp.getProcessor();
				for (int k=1;k<RS+1;k++){
					for (int kk=1;kk<CS+1;kk++){
						ImagePlus imp2 = IJ.openImage(inFol+"\\IM"+(k+((j-1)*nCom))+"_"+(kk+((jj-1)*nCom))+".tif");
						ImageProcessor impP2=imp2.getProcessor(); // we use imageProcessors to copy the data since the Imageplus objects don't handle copying properly
						impP.insert(impP2,(kk-1)*900, (k-1)*900);
					}
				}
				if (!Ren){
					CenterName="Mosaic";
				}
				IJ.saveAs(imp, "Tiff", ""+outFol+"\\"+CenterName+j+"_"+jj+".tif");
			}
		}
	}

	void RenFile(int FolNum){
		int rowN, colN;
		tFol=FolArray.get(FolNum);
		renFol=tFol+"\\renamed";
		new File(tFol+"\\renamed").mkdir();
		File tempFol= new File(tFol);
		File[] LOF=tempFol.listFiles();
		for (int ii=0;ii<LOF.length;ii++){
			String tFileName=LOF[ii].toString();
			if ((tFileName.toLowerCase()).endsWith(".tif")){
				int posRen=tFileName.indexOf(")_p");
				String tNewName=tFileName.substring(posRen+3,tFileName.length()-4);
				CenterName=tFileName.substring(posRen+3);
	
				int FR = Integer.parseInt(tNewName)%Col;
				if (((Integer.parseInt(tNewName)-FR)/Col)%2==0){
					rowN=((Integer.parseInt(tNewName)-FR)/Col)+1;
					colN=FR+1;
				} else {
					rowN=((Integer.parseInt(tNewName)-FR)/Col)+1;
					colN=Col-FR;
				}
				File newFile = new File(tFol +"\\renamed\\IM"+rowN+"_"+colN+".tif");
				boolean suc = LOF[ii].renameTo(newFile);
				int renC=0;
				while (!suc){
					newFile = new File(tFol +"\\renamed\\IM"+rowN+"_"+colN+".tif");
					renC++;
					if (renC>5){
						suc=true;
						RenError=true;
					}
				}
			}
		}
	}

	void SelFilter(){
		if (Filter){
			OpenDialog od = new OpenDialog("Please select the correct filter file", "");
			String directory = od.getDirectory();
        		String name = od.getFileName();
        		FilterPath = directory+name;
		} else {
			String directory= IJ.getDirectory("plugins");
			String name ="SParticle\\Base_Filterv2.txt";
			FilterPath = directory+name;
			BMT = directory+"SParticle\\BMTrue.ijm";
			BMF = directory+"SParticle\\BMFalse.ijm";
		}
	}

	void SelFolders(){
		FolArray = new ArrayList<String>();
		String Fols="";
		boolean TheEnd=false;
		while (!TheEnd){
			GenericDialog gd = new GenericDialog("Folders to analyse");
			gd.addMessage(Fols);
			gd.enableYesNoCancel("Add", "Done");
			gd.showDialog();
	
			if (gd.wasCanceled()){
	            		IJ.log("User clicked 'Cancel'");
	            		TheEnd=true;
			} else if (gd.wasOKed()) {
	            		DirectoryChooser DC = new DirectoryChooser("Select the folders you want to analyze");
	            		String tDir=DC.getDirectory();
	            		FolArray.add(tDir);
	            		Fols=Fols+tDir+"\n";
			} else {
	            		IJ.log("User clicked 'Done'");
	            		IJ.log("Now starting analysis");
	            		TheEnd=true;
			}
		}
	}

	void PelMenu (){
		ImType=new String[] {"Phase contrast","Fluorescence"};
		Col=20;
		ShedSet=200;
		LowLimit=200;
		UpLimit=1000000;
		
		GenericDialog gd = new GenericDialog("Settings");
		gd.addCheckbox("Rename Tile Images",Ren);
		gd.addMessage("Correct column number\nis essential for proper\nmosaic reconstruction");
		gd.addNumericField("-- Column count: ", Col, 0); 
		gd.addCheckbox("Build mosaic image",Com);
		gd.addCheckbox("Analyse image",Ana);
		gd.addMessage("____________________________");
		gd.addChoice("Image Type", ImType, ImType[0]);
		gd.addCheckbox("Use custom filter",Filter);
		gd.addCheckbox("Inspect results manually",Check);
		gd.addCheckbox("Open results in Excel",Excel);
		gd.addCheckbox("Save with outlines",Outlines);
		gd.addCheckbox("Remove out of focus objects",FocusCheck);
		gd.addStringField("Pixel size in nanometers","778", 10);
		gd.addNumericField("Maximum Watershed length: ", ShedSet, 0, 12,"Micrometers"); 
		gd.addNumericField("Minimum object size: ", LowLimit, 0, 12, "Pixels");
		gd.addNumericField("Maximum object size: ", UpLimit, 0, 12, "Pixels");
		gd.addCheckbox("Store rejected data",Reject);
		gd.showDialog();
		
		//--------Getting the values------------------------
		Ren=gd.getNextBoolean();
		Com=gd.getNextBoolean();
		Ana=gd.getNextBoolean();
		Filter=gd.getNextBoolean();
		Check=gd.getNextBoolean();
		Excel=gd.getNextBoolean();
		Outlines=gd.getNextBoolean();
		FocusCheck=gd.getNextBoolean();

		Col=(int)gd.getNextNumber();
		ShedSet=(int)gd.getNextNumber();
		
		LowLimit=(int)gd.getNextNumber();
		UpLimit=(int)gd.getNextNumber();

		ImTypeIndex=gd.getNextChoiceIndex();
		
		NanPix=gd.getNextString();
		while (NanPix.length()<3){
			NanPix ="0"+NanPix;
		}
		String pixVal="0."+NanPix;
		ShedSet=(int)(ShedSet/(Double.parseDouble(pixVal)));
		if (gd.wasCanceled()){
			skip=true;
		}
	}
}


//________________

class FractalBoxCounterForMe implements ij.plugin.filter.PlugInFilter {
	static String sizes = "2,3,4,6,8,12,16,32,64";
	static boolean blackBackground;
	int[] boxSizes;
	float[] boxCountSums;
	int maxBoxSize;
	int[] counts;
	Rectangle roi;
	int foreground;
	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_8G+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		ImagePlus imp =new ImagePlus("test",ip);
		GenericDialog gd = new GenericDialog("Fractal Box Counter", IJ.getInstance());
		gd.addStringField("Box Sizes:", sizes, 20);
		gd.addCheckbox("Black Background", blackBackground);

		//gd.showDialog();
		if (gd.wasCanceled())
			return;

		String s = gd.getNextString();

		blackBackground = gd.getNextBoolean ();

		if (s.equals(""))
			return;
		boxSizes = s2ints(s);
		if (boxSizes==null || boxSizes.length<1)
			return;
		boxCountSums = new float[boxSizes.length];
		sizes = s;
		for (int i=0; i<boxSizes.length; i++)
			maxBoxSize = Math.max(maxBoxSize, boxSizes[i]);
		counts = new int[maxBoxSize*maxBoxSize+1];
		//imp.deleteRoi();
		if (!ip.isBinary()) {
			IJ.error("8-bit binary image (0 and 255) required.");
			return;
		}
		if (blackBackground)
			foreground = 255;
		else
			foreground = 0;
		if (ip.isInvertedLut())
			foreground = 255 - foreground;
		doBoxCounts(ip);
		//IJ.register(FractalBoxCounter.class);  // keeps this class from being GC'd on 1.1 JVMs
		return;
	}

	/** Breaks the specified string into an array
	 of ints. Returns null if there is an error.*/
	public int[] s2ints(String s) {
		StringTokenizer st = new StringTokenizer(s, ", \t");
		int nInts = st.countTokens();
		int[] ints = new int[nInts];
		for(int i=0; i<nInts; i++) {
			try {ints[i] = Integer.parseInt(st.nextToken());}
			catch (NumberFormatException e) {IJ.log(""+e); return null;}
		}
		return ints;
	}

	boolean FindMargins(ImageProcessor ip) {
		if (IJ.debugMode) IJ.log("FindMargins");
		int[] histogram = new int[256];
		int width = ip.getWidth();
		int height = ip.getHeight();
		int left, right, top, bottom;

		//Find left edge
 		left = -1;
		do {
			left++;
			if (left>=width) {
				IJ.error("No non-backround pixels found.");
				return false;
			}
			ip.setRoi(left, 0, 1, height);
			histogram = ip.getHistogram();
		} while (histogram[foreground]==0);

		//Find top edge
		top = -1;
		do {
			top++;
			ip.setRoi(left, top, width-left, 1);
			histogram = ip.getHistogram();
		} while (histogram[foreground]==0);

		//Find right edge
		right =width+1;
		do {
			right--;
			ip.setRoi(right-1, top, 1, height-top);
			histogram = ip.getHistogram();
		} while (histogram[foreground]==0);

		//Find bottom edge
		bottom =height+1;
		do {
			bottom--;
			ip.setRoi(left, bottom-1, right-left, 1);
			histogram = ip.getHistogram();
		} while (histogram[foreground]==0);

		roi = new Rectangle(left, top, right-left, bottom-top);
		return true;
	}

	int count(int size, ImageProcessor ip) {
		int[] histogram = new int[256];
		int count;
		int x = roi.x;
		int y = roi.y;
		int w = (size<=roi.width)?size:roi.width;
		int h = (size<=roi.height)?size:roi.height;
		int right = roi.x+roi.width;
		int bottom = roi.y+roi.height;
		int maxCount = size*size;
		
		for (int i=1; i<=maxCount; i++)
			counts[i] = 0;
		boolean done = false;
		do {
			ip.setRoi(x, y, w, h);
			histogram = ip.getHistogram();
			counts[histogram[foreground]]++;
			x+=size;
			if (x+size>=right) {
				w = right-x;
				if (x>=right) {
					w = size;
					x = roi.x;
					y += size;
					if (y+size>=bottom)
						h = bottom-y;
					done = y>=bottom;
				}
			}
		} while (!done);
		int boxSum = 0;
		int nBoxes;
		for (int i=1; i<=maxCount; i++) {
			nBoxes = counts[i];
			if (nBoxes!=0)
				boxSum += nBoxes;
		}
		return boxSum;
	}
	
	double plot() {
		int n = boxSizes.length;
		float[] sizes = new float[boxSizes.length];
		for (int i=0; i<n; i++)
			sizes[i] = (float)Math.log(boxSizes[i]);
		CurveFitter cf = new CurveFitter(Tools.toDouble(sizes), Tools.toDouble(boxCountSums));
		cf.doFit(CurveFitter.STRAIGHT_LINE);
		double[] p = cf.getParams();
		double D = -p[1];
		String label = "D="+IJ.d2s(D,4);
		float[] px = new float[100];
		float[] py = new float[100];
		double[] a = Tools.getMinMax(sizes);
		double xmin=a[0], xmax=a[1]; 
		a = Tools.getMinMax(boxCountSums);
		double ymin=a[0], ymax=a[1]; 
		double inc = (xmax-xmin)/99.0;
		double tmp = xmin;
		for (int i=0; i<100; i++) {
			px[i]=(float)tmp;
			tmp += inc;
		}
		for (int i=0; i<100; i++)
			py[i] = (float)cf.f(p, px[i]);
		a = Tools.getMinMax(py);
		ymin = Math.min(ymin, a[0]);
		ymax = Math.max(ymax, a[1]);
		Plot plot = new Plot("Plot", "log (box size)", "log (count)", px, py);
		plot.setLimits(xmin,xmax,ymin,ymax);
		plot.addPoints(sizes, boxCountSums, PlotWindow.CIRCLE);
		plot.addLabel(0.8, 0.2, label);
		//plot.show();
		return D;			
	}

	void doBoxCounts(ImageProcessor ip) {

		if (!FindMargins(ip))
			return;
		ResultsTable rt=ResultsTable.getResultsTable();
		rt.incrementCounter();
		rt.setLabel("temp", rt.getCounter()-1);
		for (int i=0; i<boxSizes.length; i++) {
			int boxSum = count(boxSizes[i], ip);
			rt.addValue("C"+boxSizes[i], boxSum);
			boxCountSums[i] = (float)Math.log(boxSum);
		}
		double D = plot();
		rt.addValue("D", D);
		//rt.show("Results");
	}
}